package cn.superdesk.app.inventory.controller;

import cn.superdesk.app.base.model.LoginSession;
import cn.superdesk.app.base.model.PageResult;
import cn.superdesk.app.base.model.SelectOptGroup;
import cn.superdesk.app.inventory.model.param.InventoryCheckMergeSubmitParam;
import cn.superdesk.app.inventory.model.param.InventoryCheckPageQueryParam;
import cn.superdesk.app.inventory.model.param.InventoryCheckParam;
import cn.superdesk.app.inventory.model.vo.InventoryCheckSheetDetailInfoVO;
import cn.superdesk.app.inventory.model.vo.InventoryCheckSheetVO;
import cn.superdesk.app.inventory.service.InventoryCheckService;
import cn.superdesk.app.inventory.util.AdminUtils;
import cn.superdesk.libs.springweb.model.WrapperResponse;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 盘点管理 web端
 * @author xc
 * @since 2023/11/21 17:36
 */
@RestController
@RequiredArgsConstructor
@Slf4j
@RequestMapping("/web/check")
public class InventoryCheckWebController {
    private final InventoryCheckService inventoryCheckService;

    /**
     * 获取盘点单状态下拉列表
     * @return 操作类型
     */
    @GetMapping("/state")
    public WrapperResponse<List<SelectOptGroup>> getState() {
        return new WrapperResponse<>(200, "查询成功", inventoryCheckService.getState());
    }

    /**
     * 新增盘点单
     * @param param 盘点单参数
     * @return WrapperResponse
     */
    @PostMapping
    public WrapperResponse<String> addCheck(@RequestBody @Validated InventoryCheckParam param) {
        log.info("新增盘点单 ============  param : {}", JSON.toJSONString(param));
        LoginSession loginSession = AdminUtils.hasPermission(param.getOrgId());
        param.setLoginSession(loginSession);
        if (param.getTemporarySaving() == 1) {
            // 临时保存
            inventoryCheckService.saveInventoryCheckSheet(param);
        } else {
            // 保存并提交
            Integer sheetId = inventoryCheckService.saveInventoryCheckSheet(param);
            inventoryCheckService.submitInventoryCheckSheet(sheetId);
        }
        return new WrapperResponse<>(200, "操作成功");
    }

    /**
     * 分页查询盘点单记录
     * @param param 查询参数
     * @return 盘点单记录
     */
    @PostMapping("/list")
    public PageResult<InventoryCheckSheetVO> list(@RequestBody @Validated InventoryCheckPageQueryParam param) {
        log.info("PC端查询盘点单记录 param: {}", JSON.toJSONString(param));
        return inventoryCheckService.list(param);
    }


    /**
     * 查询盘点单详情
     * @param docNo 单据号
     * @return 盘点单详情
     */
    @GetMapping("/{docNo}")
    public WrapperResponse<InventoryCheckSheetDetailInfoVO> detail(@PathVariable String docNo) {
        log.info("PC端查询盘点单记录 单据号: {}", docNo);
        return new WrapperResponse<>(200, "查询成功", inventoryCheckService.detail(docNo));
    }

    /**
     * 删除临时保存盘点单
     * @param docNo 单据号
     * @return WrapperResponse
     */
    @PostMapping("/delete/{docNo}")
    public WrapperResponse<String> delete(@PathVariable String docNo) {
        log.info("删除临时保存盘点单 ============  单据号 : {}", JSON.toJSONString(docNo));
        inventoryCheckService.deleteInventoryCheckSheet(docNo);
        return new WrapperResponse<>(200, "操作成功");
    }


    /**
     * 合并提交盘点单
     * @param param 盘点单参数
     * @return WrapperResponse
     */
    @PostMapping("/mergeSubmit")
    public WrapperResponse<String> mergeSubmit(@RequestBody @Validated InventoryCheckMergeSubmitParam param) {
        log.info("合并提交盘点单 ============  param : {}", JSON.toJSONString(param));
        inventoryCheckService.mergeSubmitInventoryCheckSheet(param);
        return new WrapperResponse<>(200, "操作成功");
    }

    /**
     * 导出盘点单
     * @param docNo 盘点单单号
     * @return WrapperResponse
     */
    @PostMapping("/export/{docNo}")
    public WrapperResponse<String> export(@PathVariable("docNo") String docNo) {
        log.info("导出盘点单 == 单号: {}", docNo);
        return inventoryCheckService.export(docNo);

    }
}
