package cn.superdesk.app.inventory.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.superdesk.app.base.LoginContext;
import cn.superdesk.app.base.model.LoginSession;
import cn.superdesk.app.base.model.PageResult;
import cn.superdesk.app.base.model.SelectOptGroup;
import cn.superdesk.app.export.consts.function.fwzc.ExportInventoryFunctionConstants;
import cn.superdesk.app.export.enums.ExportTaskEnum;
import cn.superdesk.app.export.feign.ExportFeignClient;
import cn.superdesk.app.export.param.ExportTaskSubmitParam;
import cn.superdesk.app.inventory.common.enums.InventoryCheckSheetLevelEnum;
import cn.superdesk.app.inventory.common.enums.InventoryCheckSheetStateEnum;
import cn.superdesk.app.inventory.dao.entity.InventoryCheckSheetDetailEntity;
import cn.superdesk.app.inventory.dao.entity.InventoryCheckSheetEntity;
import cn.superdesk.app.inventory.dao.entity.InventoryCheckSheetMergeHistoryEntity;
import cn.superdesk.app.inventory.dao.entity.ProductInventoryEntity;
import cn.superdesk.app.inventory.dao.mapper.InventoryCheckSheetDetailEntityMapper;
import cn.superdesk.app.inventory.dao.mapper.InventoryCheckSheetEntityMapper;
import cn.superdesk.app.inventory.dao.mapper.InventoryCheckSheetMergeHistoryEntityMapper;
import cn.superdesk.app.inventory.dao.mapper.ProductInventoryEntityMapper;
import cn.superdesk.app.inventory.model.dto.MerchantVO;
import cn.superdesk.app.inventory.model.dto.OrganizationEntityVO;
import cn.superdesk.app.inventory.model.param.InventoryCheckDetailParam;
import cn.superdesk.app.inventory.model.param.InventoryCheckMergeSubmitParam;
import cn.superdesk.app.inventory.model.param.InventoryCheckPageQueryParam;
import cn.superdesk.app.inventory.model.param.InventoryCheckParam;
import cn.superdesk.app.inventory.model.vo.InventoryCheckSheetDetailInfoVO;
import cn.superdesk.app.inventory.model.vo.InventoryCheckSheetDetailVO;
import cn.superdesk.app.inventory.model.vo.InventoryCheckSheetVO;
import cn.superdesk.app.inventory.service.feign.OrganizationFeignClient;
import cn.superdesk.app.inventory.util.AdminUtils;
import cn.superdesk.app.inventory.util.DocNoUtils;
import cn.superdesk.libs.common.SuperdeskBaseException;
import cn.superdesk.libs.springweb.model.WrapperResponse;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author xc
 * @since 2023/11/21 17:36
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class InventoryCheckService {
    private final InventoryCheckSheetEntityMapper inventoryCheckSheetEntityMapper;
    private final InventoryCheckSheetDetailEntityMapper inventoryCheckSheetDetailEntityMapper;
    private final ProductInventoryEntityMapper productInventoryEntityMapper;
    private final OrganizationFeignClient organizationFeignClient;
    private final InventoryCheckSheetMergeHistoryEntityMapper inventoryCheckSheetMergeHistoryEntityMapper;
    private final ExportFeignClient exportFeignClient;

    /**
     * 保存盘点单
     * @param param 盘点单参数
     * @return 盘点单id
     */
    @Transactional(rollbackFor = Exception.class)
    public Integer saveInventoryCheckSheet(InventoryCheckParam param) {
        LoginSession loginSession = param.getLoginSession();
        List<InventoryCheckDetailParam> detailList = param.getDetailList();
        // 检查操作商品是否重复,如有重复 需要合并重复项的实际库存数量
        List<Integer> collect = detailList.stream().filter(p -> p.getEnabled() == 1).map(InventoryCheckDetailParam::getProductId).collect(Collectors.toList());
        List<InventoryCheckDetailParam> mergeList;
        HashSet<Integer> productIdSet = new HashSet<>(collect);
        if (collect.size() != productIdSet.size()) {
            log.info("商品明细存在重复商品，需要合并重复项的实际库存数量");
            mergeList = new ArrayList<>();
            Map<Integer, List<InventoryCheckDetailParam>> groupByProductId = detailList.stream().collect(Collectors.groupingBy(InventoryCheckDetailParam::getProductId));

            for (Map.Entry<Integer, List<InventoryCheckDetailParam>> entry : groupByProductId.entrySet()) {
                List<InventoryCheckDetailParam> value = entry.getValue();
                InventoryCheckDetailParam inventoryCheckDetailParam = value.get(0);
                if (value.size() > 1) {
                    Integer actualQuantityMerged = value.stream().map(InventoryCheckDetailParam::getActualQuantity).reduce(0, Integer::sum);
                    String remarkMerged = value.stream().map(InventoryCheckDetailParam::getRemark).filter(StrUtil::isNotBlank).collect(Collectors.joining(","));
                    mergeList.add(
                            inventoryCheckDetailParam.toBuilder()
                                    .actualQuantity(actualQuantityMerged)
                                    .remark(remarkMerged)
                                    .build()
                    );
                } else {
                    mergeList.add(inventoryCheckDetailParam);
                }
            }
        } else {
            mergeList = detailList;
        }
        /*
         * 校验所有明细商品所属商户是否和盘点单所属商户一致
         */
        Set<Integer> merchantIdSet = productInventoryEntityMapper.selectMerchantSetByProductIdSet(productIdSet);
        if (CollUtil.isEmpty(merchantIdSet) || merchantIdSet.size() > 1 || !merchantIdSet.contains(param.getMerchantId())) {
            log.info("商品明细归属商户id: {}, 主体商户id: {} 盘点商品不属于所选商户，不能变更商户！", merchantIdSet, param.getMerchantId());
            throw new SuperdeskBaseException(500, "盘点商品不属于所选商户，不能变更商户！");
        }

        OrganizationEntityVO projectData;
        WrapperResponse<OrganizationEntityVO> projectInfoResponse = organizationFeignClient.getProjectInfoById(param.getOrgId());
        if (200 == projectInfoResponse.getCode()) {
            projectData = projectInfoResponse.getData();
        } else {
            throw new SuperdeskBaseException(500, "项目信息查询异常");
        }
        /*
         * 保存盘点单
         */
        InventoryCheckSheetEntity inventoryCheckSheetEntity;
        // 主动查询一次商户名称
        String merchantName;
        WrapperResponse<MerchantVO> merchantVOWrapperResponse = organizationFeignClient.getMerchantParamById(param.getMerchantId().toString());
        log.info("查询商户信息结果: {}", JSON.toJSONString(merchantVOWrapperResponse));
        if (merchantVOWrapperResponse != null && 200 == merchantVOWrapperResponse.getCode() && merchantVOWrapperResponse.getData() != null) {
            merchantName = merchantVOWrapperResponse.getData().getName();
        }else {
            merchantName = param.getMerchantName();
        }

        if (param.getId() == null) {
            // 新增盘点单
            InventoryCheckSheetEntity.InventoryCheckSheetEntityBuilder builder = InventoryCheckSheetEntity.builder();


            // 生成单据号
            String docNo = DocNoUtils.generateDocNo("PD");

            inventoryCheckSheetEntity = builder
                    .compId(Integer.parseInt(projectData.getCompanyIds()))
                    .compName(projectData.getCompanyIdsName())
                    .orgId(param.getOrgId())
                    .orgName(projectData.getName())
                    .docNo(docNo)
                    .merchantId(param.getMerchantId())
                    .merchantName(merchantName)
                    .productTypeCount(productIdSet.size())
                    .gainAndLossQuantity(mergeList.stream()
                            .filter(p -> p.getEnabled() == 1)
                            .map(p -> p.getActualQuantity() - p.getOriginalQuantity())
                            .reduce(0, Integer::sum))
                    .remark(param.getRemark())
                    .state(InventoryCheckSheetStateEnum.临时保存.getCode())
                    .createTime(new Date())
                    .createUserName(loginSession.getRealName())
                    .createBy(loginSession.getUserId())
                    .build();
            inventoryCheckSheetEntityMapper.insertSelective(inventoryCheckSheetEntity);
            // 保存明细
            List<InventoryCheckSheetDetailEntity> detailEntityList = new ArrayList<>();
            for (InventoryCheckDetailParam inventoryCheckDetailParam : mergeList) {
                BigDecimal costPrice = productInventoryEntityMapper.selectCostPriceById(inventoryCheckDetailParam.getProductId());
                if (costPrice == null) {
                    costPrice = BigDecimal.ZERO;
                }
                detailEntityList.add(
                        InventoryCheckSheetDetailEntity.builder()
                                .docNo(docNo)
                                .productId(inventoryCheckDetailParam.getProductId())
                                .productName(inventoryCheckDetailParam.getProductName())
                                .productCode(inventoryCheckDetailParam.getProductCode())
                                .unit(inventoryCheckDetailParam.getUnit())
                                .originalQuantity(inventoryCheckDetailParam.getOriginalQuantity())
                                .actualQuantity(inventoryCheckDetailParam.getActualQuantity())
                                .gainAndLossQuantity(inventoryCheckDetailParam.getActualQuantity() - inventoryCheckDetailParam.getOriginalQuantity())
                                .costPrice(costPrice)
                                .costPriceTotal(costPrice.multiply(BigDecimal.valueOf(inventoryCheckDetailParam.getActualQuantity())))
                                .remark(inventoryCheckDetailParam.getRemark())
                                .enabled(inventoryCheckDetailParam.getEnabled())
                                .build()
                );
            }
            inventoryCheckSheetDetailEntityMapper.insertList(detailEntityList);
        } else {
            // 修改盘点单
            inventoryCheckSheetEntity = inventoryCheckSheetEntityMapper.selectByPrimaryKey(param.getId());
            if (inventoryCheckSheetEntity == null) {
                throw new SuperdeskBaseException(500, "盘点单不存在");
            }
            if (!InventoryCheckSheetStateEnum.临时保存.getCode().equals(inventoryCheckSheetEntity.getState())) {
                throw new SuperdeskBaseException(500, "只有临时保存状态的盘点单才能修改");
            }

            inventoryCheckSheetEntity = inventoryCheckSheetEntity.toBuilder()
                    .compId(Integer.parseInt(projectData.getCompanyIds()))
                    .compName(projectData.getCompanyIdsName())
                    .orgId(param.getOrgId())
                    .orgName(projectData.getName())
                    .productTypeCount(productIdSet.size())
                    .gainAndLossQuantity(mergeList.stream()
                            .filter(p -> p.getEnabled() == 1)
                            .map(p -> p.getActualQuantity() - p.getOriginalQuantity())
                            .reduce(0, Integer::sum))
                    .remark(param.getRemark())
                    .state(InventoryCheckSheetStateEnum.临时保存.getCode())
                    .updateTime(new Date())
                    .updateUserName(loginSession.getUserName())
                    .updateBy(loginSession.getUserId())
                    .build();
            inventoryCheckSheetEntityMapper.updateByPrimaryKeySelective(inventoryCheckSheetEntity);
            // 保存明细
            List<InventoryCheckSheetDetailEntity> detailEntityInsertList = new ArrayList<>();
            List<InventoryCheckSheetDetailEntity> detailEntityUpdateList = new ArrayList<>();
            List<InventoryCheckDetailParam> insertList = mergeList.stream().filter(p -> p.getId() == null).collect(Collectors.toList());
            List<InventoryCheckDetailParam> updateList = mergeList.stream().filter(p -> p.getId() != null).collect(Collectors.toList());

            for (InventoryCheckDetailParam inventoryCheckDetailParam : insertList) {
                BigDecimal costPrice = productInventoryEntityMapper.selectCostPriceById(inventoryCheckDetailParam.getProductId());
                if (costPrice == null) {
                    costPrice = BigDecimal.ZERO;
                }
                detailEntityInsertList.add(
                        InventoryCheckSheetDetailEntity.builder()
                                .docNo(inventoryCheckSheetEntity.getDocNo())
                                .productId(inventoryCheckDetailParam.getProductId())
                                .productName(inventoryCheckDetailParam.getProductName())
                                .productCode(inventoryCheckDetailParam.getProductCode())
                                .unit(inventoryCheckDetailParam.getUnit())
                                .originalQuantity(inventoryCheckDetailParam.getOriginalQuantity())
                                .actualQuantity(inventoryCheckDetailParam.getActualQuantity())
                                .gainAndLossQuantity(inventoryCheckDetailParam.getActualQuantity() - inventoryCheckDetailParam.getOriginalQuantity())
                                .remark(inventoryCheckDetailParam.getRemark())
                                .costPrice(costPrice)
                                .costPriceTotal(costPrice.multiply(BigDecimal.valueOf(inventoryCheckDetailParam.getActualQuantity())))
                                .enabled(inventoryCheckDetailParam.getEnabled())
                                .build()
                );
            }

            for (InventoryCheckDetailParam inventoryCheckDetailParam : updateList) {
                BigDecimal costPrice = productInventoryEntityMapper.selectCostPriceById(inventoryCheckDetailParam.getProductId());
                if (costPrice == null) {
                    costPrice = BigDecimal.ZERO;
                }
                detailEntityUpdateList.add(
                        InventoryCheckSheetDetailEntity.builder()
                                .id(inventoryCheckDetailParam.getId())
                                .docNo(inventoryCheckSheetEntity.getDocNo())
                                .productId(inventoryCheckDetailParam.getProductId())
                                .productName(inventoryCheckDetailParam.getProductName())
                                .productCode(inventoryCheckDetailParam.getProductCode())
                                .unit(inventoryCheckDetailParam.getUnit())
                                .originalQuantity(inventoryCheckDetailParam.getOriginalQuantity())
                                .actualQuantity(inventoryCheckDetailParam.getActualQuantity())
                                .gainAndLossQuantity(inventoryCheckDetailParam.getActualQuantity() - inventoryCheckDetailParam.getOriginalQuantity())
                                .remark(inventoryCheckDetailParam.getRemark())
                                .costPrice(costPrice)
                                .costPriceTotal(costPrice.multiply(BigDecimal.valueOf(inventoryCheckDetailParam.getActualQuantity())))
                                .enabled(inventoryCheckDetailParam.getEnabled())
                                .build()
                );
            }

            if (CollUtil.isNotEmpty(detailEntityInsertList)) {
                inventoryCheckSheetDetailEntityMapper.insertList(detailEntityInsertList);
            }

            if (CollUtil.isNotEmpty(detailEntityUpdateList)) {
                for (InventoryCheckSheetDetailEntity inventoryCheckSheetDetailEntity : detailEntityUpdateList) {
                    inventoryCheckSheetDetailEntityMapper.updateByPrimaryKeySelective(inventoryCheckSheetDetailEntity);
                }
            }

        }

        return inventoryCheckSheetEntity.getId();
    }

    /**
     * 提交盘点单
     * @param sheetId 盘点单id
     */
    @Transactional(rollbackFor = Exception.class)
    public void submitInventoryCheckSheet(Integer sheetId) {
        LoginSession loginSession = LoginContext.getAndValidateLoginUser();
        log.info("提交盘点单 ============ sheetId : {}", sheetId);
        InventoryCheckSheetEntity inventoryCheckSheetEntity = inventoryCheckSheetEntityMapper.selectByPrimaryKey(sheetId);
        if (inventoryCheckSheetEntity == null) {
            throw new SuperdeskBaseException(500, "盘点单不存在");
        }
        if (!InventoryCheckSheetStateEnum.临时保存.getCode().equals(inventoryCheckSheetEntity.getState())) {
            throw new SuperdeskBaseException(500, "盘点单状态异常");
        }

        List<InventoryCheckSheetDetailEntity> inventoryCheckSheetDetailEntityList = inventoryCheckSheetDetailEntityMapper.select(
                InventoryCheckSheetDetailEntity.builder()
                        .docNo(inventoryCheckSheetEntity.getDocNo())
                        .enabled(1)
                        .build()
        );
        if (CollUtil.isEmpty(inventoryCheckSheetDetailEntityList)) {
            throw new SuperdeskBaseException(500, "盘点单明细不存在");
        }

        // 更新库存信息
        for (InventoryCheckSheetDetailEntity inventoryCheckSheetDetailEntity : inventoryCheckSheetDetailEntityList) {
            ProductInventoryEntity productInventoryEntity = productInventoryEntityMapper.selectByProductId(inventoryCheckSheetDetailEntity.getProductId());
            if (productInventoryEntity == null) {
                int i = productInventoryEntityMapper.insertSelective(
                        ProductInventoryEntity.builder()
                                .productId(inventoryCheckSheetDetailEntity.getProductId())
                                .version(0)
                                .quantity(inventoryCheckSheetDetailEntity.getActualQuantity())
                                .createdBy(loginSession.getUserId())
                                .createdName(loginSession.getRealName())
                                .createdAt(new Date())
                                .build()
                );
                if (i == 0) {
                    log.info("库存记录新增失败, 商品id :{}", inventoryCheckSheetDetailEntity.getProductId());
                    throw new SuperdeskBaseException(500, "盘点单提交异常，库存更新失败");
                }
            } else {
                // 根据商品id更新库存，必须是非多规格的商品
                int i = productInventoryEntityMapper.updateActualInventoryById(
                        productInventoryEntity.getId(),
                        inventoryCheckSheetDetailEntity.getActualQuantity(),
                        loginSession.getUserId(),
                        loginSession.getRealName()
                );
                if (i == 0) {
                    log.info("库存更新失败, 商品id :{}", inventoryCheckSheetDetailEntity.getProductId());
                    throw new SuperdeskBaseException(500, "盘点单提交异常，库存更新失败");
                }
            }

        }
        // 更新盘点单状态
        Date date = new Date();
        inventoryCheckSheetEntityMapper.updateByPrimaryKeySelective(
                InventoryCheckSheetEntity.builder()
                        .id(sheetId)
                        .state(InventoryCheckSheetStateEnum.盘点完成.getCode())
                        .finishTime(date)
                        .finishBy(loginSession.getUserId())
                        .finishUserName(loginSession.getRealName())
                        .updateTime(date)
                        .updateUserName(loginSession.getRealName())
                        .updateBy(loginSession.getUserId())
                        .build()
        );
    }

    /**
     * 分页查询盘点单列表
     * @param param 查询参数
     * @return 盘点单列表
     */
    public PageResult<InventoryCheckSheetVO> list(InventoryCheckPageQueryParam param) {
        AdminUtils.hasPermission(param.getOrgId());
        if (StrUtil.isBlank(param.getStartTime()) || StrUtil.isBlank(param.getEndTime())) {
            Date date = new Date();
            DateTime threeMonthAgo = DateUtil.offsetMonth(date, -3);
            param.setStartTime(DateUtil.format(threeMonthAgo, "yyyy-MM-dd"));
            param.setEndTime(DateUtil.format(date, "yyyy-MM-dd"));
        } else {
            DateTime startTime = DateUtil.parse(param.getStartTime(), "yyyy-MM-dd");
            DateTime endTime = DateUtil.parse(param.getEndTime(), "yyyy-MM-dd");
            if (startTime.isAfter(endTime)) {
                throw new SuperdeskBaseException(500, "开始时间不能大于结束时间");
            }
            param.setStartTime(DateUtil.format(startTime, "yyyy-MM-dd"));
            param.setEndTime(DateUtil.format(endTime, "yyyy-MM-dd"));
        }
        try (Page<InventoryCheckSheetVO> ignored = PageHelper.startPage(param.getPageNo(), param.getPageSize())) {
            List<InventoryCheckSheetVO> list = inventoryCheckSheetEntityMapper.selectList(param);
            list.forEach(
                    vo -> vo.setStateName(InventoryCheckSheetStateEnum.getDescByCode(vo.getState()))
            );
            return new PageResult<>(param.getPageNo(), param.getPageSize(), (int) ignored.getTotal(), list);
        } catch (Exception e) {
            log.error("分页查询盘点单异常", e);
            throw new SuperdeskBaseException(500, "分页查询库存盘点单据异常");
        }
    }

    /**
     * 查询盘点单详情
     * @param docNo 单据号
     * @return 盘点单详情
     */
    public InventoryCheckSheetDetailInfoVO detail(String docNo) {
        InventoryCheckSheetEntity inventoryCheckSheetEntity = inventoryCheckSheetEntityMapper.selectOne(
                InventoryCheckSheetEntity.builder().docNo(docNo).build()
        );
        if (inventoryCheckSheetEntity == null) {
            throw new SuperdeskBaseException(500, "盘点单不存在");
        }
        AdminUtils.hasPermission(inventoryCheckSheetEntity.getOrgId());

        List<InventoryCheckSheetDetailEntity> detailEntityList = inventoryCheckSheetDetailEntityMapper.select(
                InventoryCheckSheetDetailEntity.builder()
                        .enabled(1)
                        .docNo(docNo)
                        .build()
        );
        if (CollUtil.isEmpty(detailEntityList)) {
            throw new SuperdeskBaseException(500, "盘点单明细不存在");
        }

        InventoryCheckSheetVO sheetVO = InventoryCheckSheetVO.builder()
                .id(inventoryCheckSheetEntity.getId())
                .docNo(inventoryCheckSheetEntity.getDocNo())
                .orgId(inventoryCheckSheetEntity.getOrgId())
                .orgName(inventoryCheckSheetEntity.getOrgName())
                .merchantId(inventoryCheckSheetEntity.getMerchantId())
                .merchantName(inventoryCheckSheetEntity.getMerchantName())
                .remark(inventoryCheckSheetEntity.getRemark())
                .orgName(inventoryCheckSheetEntity.getOrgName())
                .createUserName(inventoryCheckSheetEntity.getCreateUserName())
                .createTime(inventoryCheckSheetEntity.getCreateTime())
                .finishUserName(inventoryCheckSheetEntity.getFinishUserName())
                .finishTime(inventoryCheckSheetEntity.getFinishTime())
                .state(inventoryCheckSheetEntity.getState())
                .stateName(InventoryCheckSheetStateEnum.getDescByCode(inventoryCheckSheetEntity.getState()))
                .build();

        List<InventoryCheckSheetDetailVO> collect;
        if (InventoryCheckSheetStateEnum.盘点完成.getCode().equals(inventoryCheckSheetEntity.getState())) {
            collect = detailEntityList.stream().map(
                    detailEntity ->
                            InventoryCheckSheetDetailVO.builder()
                                    .id(detailEntity.getId())
                                    .docNo(detailEntity.getDocNo())
                                    .productId(detailEntity.getProductId())
                                    .productName(detailEntity.getProductName())
                                    .productCode(detailEntity.getProductCode())
                                    .unit(detailEntity.getUnit())
                                    .originalQuantity(detailEntity.getOriginalQuantity())
                                    .actualQuantity(detailEntity.getActualQuantity())
                                    .actualQuantity(detailEntity.getActualQuantity())
                                    .gainAndLossQuantity(detailEntity.getGainAndLossQuantity())
                                    .costPrice(detailEntity.getCostPrice())
                                    .costPriceTotal(detailEntity.getCostPriceTotal())
                                    .remark(detailEntity.getRemark())
                                    .build()
            ).collect(Collectors.toList());
        } else {
            /*
             * 获取商品的实时库存数量
             */
            Set<Integer> productIdSet = detailEntityList.stream().map(InventoryCheckSheetDetailEntity::getProductId).collect(Collectors.toSet());
            List<ProductInventoryEntity> productInventoryEntityList = productInventoryEntityMapper.selectProductInventoryByIdSet(productIdSet);
            Map<Integer, ProductInventoryEntity> productInventoryEntityMap = productInventoryEntityList.stream()
                    .collect(Collectors.toMap(ProductInventoryEntity::getProductId, p -> p));
            collect = detailEntityList.stream().map(
                    detailEntity ->
                            InventoryCheckSheetDetailVO.builder()
                                    .id(detailEntity.getId())
                                    .docNo(detailEntity.getDocNo())
                                    .productId(detailEntity.getProductId())
                                    .productName(detailEntity.getProductName())
                                    .productCode(detailEntity.getProductCode())
                                    .unit(detailEntity.getUnit())
                                    .originalQuantity(productInventoryEntityMap.get(detailEntity.getProductId()) == null ? 0 : productInventoryEntityMap.get(detailEntity.getProductId()).getQuantity())// 原库存 数据库实时的
                                    .actualQuantity(detailEntity.getActualQuantity())// 实际库存 用户填的
                                    .actualQuantity(detailEntity.getActualQuantity())
                                    .gainAndLossQuantity(detailEntity.getGainAndLossQuantity())
                                    .costPrice(detailEntity.getCostPrice())
                                    .costPriceTotal(detailEntity.getCostPriceTotal())
                                    .remark(detailEntity.getRemark())
                                    .build()
            ).collect(Collectors.toList());
        }

        return InventoryCheckSheetDetailInfoVO.builder()
                .inventoryCheckSheetVO(sheetVO)
                .detailList(collect)
                .build();
    }

    /**
     * 获取盘点单状态
     * @return
     */
    public List<SelectOptGroup> getState() {
        List<SelectOptGroup> list = new ArrayList<>();
        for (InventoryCheckSheetStateEnum value : InventoryCheckSheetStateEnum.values()) {
            if (value.equals(InventoryCheckSheetStateEnum.已删除)) {
                continue;
            }
            list.add(new SelectOptGroup(value.getDesc(), value.getCode().toString()));
        }
        return list;
    }

    /**
     * 删除盘点单（逻辑）
     * @param docNo 单据号
     */
    @Transactional(rollbackFor = Exception.class)
    public void deleteInventoryCheckSheet(String docNo) {
        InventoryCheckSheetEntity inventoryCheckSheetEntity = inventoryCheckSheetEntityMapper.selectOne(
                InventoryCheckSheetEntity.builder()
                        .docNo(docNo)
                        .build()
        );
        if (inventoryCheckSheetEntity == null) {
            throw new SuperdeskBaseException(500, "盘点单不存在");
        }
        LoginSession loginSession = AdminUtils.hasPermission(inventoryCheckSheetEntity.getOrgId());

        if (!InventoryCheckSheetStateEnum.临时保存.getCode().equals(inventoryCheckSheetEntity.getState())) {
            throw new SuperdeskBaseException(500, "只有临时保存状态的盘点单才能删除");
        }
        inventoryCheckSheetEntityMapper.updateByPrimaryKeySelective(
                InventoryCheckSheetEntity.builder()
                        .id(inventoryCheckSheetEntity.getId())
                        .state(InventoryCheckSheetStateEnum.已删除.getCode())
                        .updateTime(new Date())
                        .updateUserName(loginSession.getRealName())
                        .updateBy(loginSession.getUserId())
                        .build()
        );
    }

    /**
     * 合并提交盘点单
     * @param param 盘点单参数
     */
    @Transactional(rollbackFor = Exception.class)
    public void mergeSubmitInventoryCheckSheet(InventoryCheckMergeSubmitParam param) {
        LoginSession loginSession = AdminUtils.hasPermission(param.getOrgId());
        Set<String> docNoSet = param.getDocNoList();
        // 根据单号查询 未删除 && 临时保存状态 盘点子单
        List<InventoryCheckSheetEntity> list = inventoryCheckSheetEntityMapper.selectEnabledTemporaryDocByDocNoList(docNoSet);
        if (CollUtil.isEmpty(list) || list.size() != docNoSet.size()) {
            throw new SuperdeskBaseException(500, "盘点单数据异常");
        }
        Set<Integer> merchantIdSet = list.stream().map(InventoryCheckSheetEntity::getMerchantId).collect(Collectors.toSet());
        if (merchantIdSet.size() > 1) {
            throw new SuperdeskBaseException(500, "盘点单所属商户不一致");
        }
        Set<Integer> orgIdSet = list.stream().map(InventoryCheckSheetEntity::getOrgId).collect(Collectors.toSet());
        if (orgIdSet.size() > 1) {
            throw new SuperdeskBaseException(500, "盘点单所属项目不一致");
        }

        // 生成合并后的单据号
        String docNo = DocNoUtils.generateDocNo("PD");
        // 新增关联关系
        List<InventoryCheckSheetMergeHistoryEntity> historyList = new ArrayList<>();
        docNoSet.forEach(
                p -> historyList.add(
                        InventoryCheckSheetMergeHistoryEntity.builder()
                                .parentSheetDocNo(docNo)
                                .mergedSheetDocNo(p)
                                .build()
                )
        );
        inventoryCheckSheetMergeHistoryEntityMapper.insertList(historyList);

        // load所有子单的明细
        List<InventoryCheckSheetDetailEntity> detailEntityList = inventoryCheckSheetDetailEntityMapper.selectByHistory(docNo);

        // 根据商品id进行合并，合并后的实际库存数量为合并前的实际库存数量之和
        Map<Integer, List<InventoryCheckSheetDetailEntity>> collect = detailEntityList.stream()
                .collect(Collectors.groupingBy(InventoryCheckSheetDetailEntity::getProductId));

        List<InventoryCheckSheetDetailEntity> mergeList = new ArrayList<>();

        // 构建合并单明细
        for (Map.Entry<Integer, List<InventoryCheckSheetDetailEntity>> entry : collect.entrySet()) {
            List<InventoryCheckSheetDetailEntity> value = entry.getValue();
            InventoryCheckSheetDetailEntity inventoryCheckSheetDetailEntity = value.get(0);
            if (value.size() > 1) {
                Integer actualQuantityMerged = value.stream().map(InventoryCheckSheetDetailEntity::getActualQuantity).reduce(0, Integer::sum);
                String remarkMerged = value.stream().map(InventoryCheckSheetDetailEntity::getRemark).filter(StrUtil::isNotBlank).collect(Collectors.joining(","));
                BigDecimal costPriceTotal = value.stream().map(InventoryCheckSheetDetailEntity::getCostPriceTotal).reduce(BigDecimal.ZERO, BigDecimal::add);
                mergeList.add(
                        InventoryCheckSheetDetailEntity.builder()
                                .docNo(docNo)
                                .productId(inventoryCheckSheetDetailEntity.getProductId())
                                .productName(inventoryCheckSheetDetailEntity.getProductName())
                                .productCode(inventoryCheckSheetDetailEntity.getProductCode())
                                .unit(inventoryCheckSheetDetailEntity.getUnit())
                                .originalQuantity(inventoryCheckSheetDetailEntity.getOriginalQuantity())
                                .actualQuantity(actualQuantityMerged)
                                .gainAndLossQuantity(actualQuantityMerged - inventoryCheckSheetDetailEntity.getOriginalQuantity())
                                .costPrice(inventoryCheckSheetDetailEntity.getCostPrice())
                                .costPriceTotal(costPriceTotal)
                                .remark(remarkMerged)
                                .enabled(1)
                                .build()
                );
            } else {
                mergeList.add(inventoryCheckSheetDetailEntity.toBuilder()
                        .id(null)
                        .docNo(docNo)
                        .build()
                );
            }
        }
        // 保存合并单明细
        inventoryCheckSheetDetailEntityMapper.insertList(mergeList);

        // 更新商品库存
        mergeList.forEach(
                p -> {
                    ProductInventoryEntity productInventoryEntity = productInventoryEntityMapper.selectByProductId(p.getProductId());
                    if (productInventoryEntity != null) {
                        productInventoryEntityMapper.updateActualInventoryByProductId(p.getProductId(), p.getActualQuantity(), loginSession.getUserId(), loginSession.getRealName());
                    } else {
                        // 如果库存信息不存在 则补全
                        productInventoryEntityMapper.insertSelective(
                                ProductInventoryEntity.builder()
                                        .productId(p.getProductId())
                                        .version(0)
                                        .quantity(p.getActualQuantity())
                                        .createdBy(loginSession.getUserId())
                                        .createdName(loginSession.getUserName())
                                        .createdAt(new Date())
                                        .build()
                        );
                    }
                }
        );
        // 统计数据
        Integer mergedGainAndLossQuantity = mergeList.stream().map(InventoryCheckSheetDetailEntity::getGainAndLossQuantity).reduce(0, Integer::sum);
        int mergedListSize = mergeList.size();


        // 更新合并的盘点单状态 并改为子单
        Date date = new Date();
        list.forEach(
                p -> inventoryCheckSheetEntityMapper.updateByPrimaryKeySelective(
                        // 更新盘点单状态
                        InventoryCheckSheetEntity.builder()
                                .id(p.getId())
                                .level(InventoryCheckSheetLevelEnum.子单.getCode())
                                .parentDocNo(docNo)
                                .state(InventoryCheckSheetStateEnum.盘点完成.getCode())
                                .finishTime(date)
                                .finishBy(loginSession.getUserId())
                                .finishUserName(loginSession.getRealName())
                                .updateTime(date)
                                .updateUserName(loginSession.getRealName())
                                .updateBy(loginSession.getUserId())
                                .build()
                )
        );

        // 保存合并后的盘点单，组织归属找第一个子单的copy过来
        InventoryCheckSheetEntity childEntity = list.get(0);
        inventoryCheckSheetEntityMapper.insertSelective(
                InventoryCheckSheetEntity.builder()
                        .compId(childEntity.getCompId())
                        .compName(childEntity.getCompName())
                        .orgId(childEntity.getOrgId())
                        .orgName(childEntity.getOrgName())
                        .docNo(docNo)
                        .level(InventoryCheckSheetLevelEnum.父单.getCode())
                        .mergedDocNo(String.join(",", docNoSet))
                        .merchantId(childEntity.getMerchantId())
                        .merchantName(childEntity.getMerchantName())
                        .productTypeCount(mergedListSize)
                        .gainAndLossQuantity(mergedGainAndLossQuantity)
                        .remark("合并盘点单")
                        .state(InventoryCheckSheetStateEnum.盘点完成.getCode())
                        .createTime(date)
                        .createUserName(loginSession.getRealName())
                        .createBy(loginSession.getUserId())
                        .finishBy(loginSession.getUserId())
                        .finishUserName(loginSession.getRealName())
                        .finishTime(date)
                        .build()
        );
    }

    /**
     * 导出盘点单
     * @param docNo 盘点单单号
     */
    public WrapperResponse<String> export(String docNo) {

        InventoryCheckSheetEntity inventoryCheckSheetEntity = inventoryCheckSheetEntityMapper.selectOne(
                InventoryCheckSheetEntity.builder()
                        .docNo(docNo)
                        .build()
        );

        if (inventoryCheckSheetEntity == null) {
            throw new SuperdeskBaseException(500, "盘点单不存在");
        }
        if (!InventoryCheckSheetStateEnum.盘点完成.getCode().equals(inventoryCheckSheetEntity.getState())) {
            throw new SuperdeskBaseException(500, "盘点单未完成，无法导出");
        }

        LoginSession loginSession = AdminUtils.hasPermission(inventoryCheckSheetEntity.getOrgId());

        HashMap<String, Object> map = new HashMap<>();
        map.put("docNo", docNo);

        WrapperResponse<String> submit = exportFeignClient.submit(ExportTaskSubmitParam.builder()
                .platform(ExportTaskEnum.Platform.楼闪送.getCode())
                .module(ExportTaskEnum.Module.盘点.getCode())
                .function(ExportInventoryFunctionConstants.导出盘点单)
                .moduleName(ExportTaskEnum.Module.盘点.getDesc())
                .source(ExportTaskEnum.Module.盘点.getDesc())
                .exportFormat(ExportTaskEnum.Format.XLSX.getCode())
                .param(JSON.toJSONString(map))
                .artifactName("盘点记录" + DateUtil.format(new Date(), "yyyyMMddHHmmss"))
                .createUserId(loginSession.getUserId())
                .createUserName(loginSession.getUserName())
                .compress(1)
                .build());
        log.info("提交盘点单导出任务结果：{}，docNo：{}", JSON.toJSONString(submit), docNo);
        if (submit.getCode() != 200) {
            throw new SuperdeskBaseException(500, "导出任务提交失败");
        } else {
            return new WrapperResponse<>(200, "导出任务提交成功");
        }
    }
}
