package cn.superdesk.app.inventory.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.superdesk.app.base.LoginContext;
import cn.superdesk.app.base.model.LoginSession;
import cn.superdesk.app.base.model.PageResult;
import cn.superdesk.app.base.model.SelectOptGroup;
import cn.superdesk.app.export.consts.function.fwzc.ExportInventoryFunctionConstants;
import cn.superdesk.app.export.enums.ExportTaskEnum;
import cn.superdesk.app.export.enums.lss.export.InventoryOperationTypeEnum;
import cn.superdesk.app.export.feign.ExportFeignClient;
import cn.superdesk.app.export.param.ExportTaskSubmitParam;
import cn.superdesk.app.inventory.dao.entity.InventoryOperationDetailEntity;
import cn.superdesk.app.inventory.dao.entity.InventoryOperationEntity;
import cn.superdesk.app.inventory.dao.entity.ProductInventoryEntity;
import cn.superdesk.app.inventory.dao.mapper.InventoryOperationDetailEntityMapper;
import cn.superdesk.app.inventory.dao.mapper.InventoryOperationEntityMapper;
import cn.superdesk.app.inventory.dao.mapper.ProductInventoryEntityMapper;
import cn.superdesk.app.inventory.model.dto.InventoryOperationExcelImportDTO;
import cn.superdesk.app.inventory.model.dto.OrganizationEntityVO;
import cn.superdesk.app.inventory.model.dto.ProductInventoryDTO;
import cn.superdesk.app.inventory.model.param.InventoryOperationDetailParam;
import cn.superdesk.app.inventory.model.param.InventoryOperationImportParam;
import cn.superdesk.app.inventory.model.param.InventoryOperationPageQueryParam;
import cn.superdesk.app.inventory.model.param.InventoryOperationParam;
import cn.superdesk.app.inventory.model.vo.InventoryOperationDetailInfoVO;
import cn.superdesk.app.inventory.model.vo.InventoryOperationDetailVO;
import cn.superdesk.app.inventory.model.vo.InventoryOperationVO;
import cn.superdesk.app.inventory.service.feign.OrganizationFeignClient;
import cn.superdesk.app.inventory.util.AdminUtils;
import cn.superdesk.app.inventory.util.DocNoUtils;
import cn.superdesk.libs.common.SuperdeskBaseException;
import cn.superdesk.libs.common2.excel.ExcelReader;
import cn.superdesk.libs.springweb.model.WrapperResponse;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author xc
 * @since 2023/11/20 15:59
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class InventoryOperationService {
    private final ProductInventoryEntityMapper productInventoryEntityMapper;
    private final InventoryOperationEntityMapper inventoryOperationEntityMapper;
    private final InventoryOperationDetailEntityMapper inventoryOperationDetailEntityMapper;
    private final OrganizationFeignClient organizationFeignClient;
    private final ExportFeignClient exportFeignClient;

    /**
     * 获取操作类型
     * @return 操作类型
     */
    public List<SelectOptGroup> getOperationType() {
        List<SelectOptGroup> list = new ArrayList<>();
        for (InventoryOperationTypeEnum value : InventoryOperationTypeEnum.values()) {
            SelectOptGroup selectOptGroup = new SelectOptGroup(value.getDesc(), value.getCode().toString());
            list.add(selectOptGroup);
        }
        return list;
    }

    /**
     * 操作库存
     * @param param 操作库存参数
     */
    @Transactional(rollbackFor = Exception.class)
    public void operateInventory(InventoryOperationParam param) {
        LoginSession loginUser = LoginContext.getAndValidateLoginUser();
        if (Objects.equals(param.getSourceWarehouseId(), param.getTargetWarehouseId())) {
            throw new SuperdeskBaseException(500, "调出仓库和调入仓库不能相同");
        }
        List<InventoryOperationDetailParam> detailList = param.getDetailList();

        // 检查操作商品是否重复
        List<Integer> productIdList = detailList.stream().map(InventoryOperationDetailParam::getProductId).collect(Collectors.toList());
        Set<Integer> productIdSet = new HashSet<>(productIdList);
        int size = productIdSet.size();
        if (size != productIdList.size()) {
            throw new RuntimeException("操作商品重复");
        }
        // 检查商品库存信息是否完整（排除多规格商品）
        List<ProductInventoryEntity> inventoryList = productInventoryEntityMapper.selectProductInventoryByIdSet(productIdSet);
        if (CollUtil.isEmpty(inventoryList) || inventoryList.size() != size) {
            log.info("商品库存信息不完整,需要补足商品库存记录");
            List<Integer> existProductIdList = inventoryList.stream().map(ProductInventoryEntity::getProductId).collect(Collectors.toList());
            HashSet<Integer> copySet = new HashSet<>(productIdSet);
            existProductIdList.forEach(copySet::remove);
            log.info("需要补足库存信息的商品id集合: {}", copySet);
            List<ProductInventoryEntity> insertList = new ArrayList<>();
            for (Integer productId : copySet) {
                insertList.add(ProductInventoryEntity.builder()
                        .productId(productId)
                        .version(0)
                        .quantity(0)
                        .createdBy(loginUser.getUserId())
                        .createdName(loginUser.getUserName())
                        .createdAt(new Date())
                        .build());
            }
            productInventoryEntityMapper.insertList(insertList);
            // 补足后重新查询
            inventoryList = productInventoryEntityMapper.selectProductInventoryByIdSet(productIdSet);
        }
        // 转化为map key：商品id value：变动前库存数量
        Map<Integer, Integer> inventoryMap = inventoryList.stream()
                .collect(Collectors.toMap(ProductInventoryEntity::getProductId, ProductInventoryEntity::getQuantity));

        // 构建库存操作单据主体信息
        InventoryOperationEntity.InventoryOperationEntityBuilder builder = getInventoryOperationBuilder(param.getOrgId());

        Map<Integer, Integer> collect = detailList.stream().collect(Collectors.toMap(
                InventoryOperationDetailParam::getProductId,
                InventoryOperationDetailParam::getInventoryChangeQuantity
        ));
        boolean abs = abs(param.getOperationType());
        // 更新库存
        for (ProductInventoryEntity productInventoryEntity : inventoryList) {
            Integer change = collect.get(productInventoryEntity.getProductId());
            if (change == null) {
                throw new SuperdeskBaseException(500, "商品库存变动数异常");
            }
            if (abs) {
                change = Math.abs(change);
            } else {
                change = -Math.abs(change);
            }

            int effect = productInventoryEntityMapper.updateInventory(productInventoryEntity.getId(), change, loginUser.getUserId(), loginUser.getUserName());
            if (effect != 1) {
                throw new SuperdeskBaseException(500, "商品库存更新异常,商品id: " + productInventoryEntity.getProductId());
            }
        }

        // 生成单据号
        String docNo = DocNoUtils.generateDocNo("KC");

        // 插入库存操作单据主体信息
        InventoryOperationEntity inventoryOperationEntity = builder.docNo(docNo)
                .merchantId(param.getMerchantId())
                .merchantName(param.getMerchantName())
                .operationType(param.getOperationType())
                .productTypeCount(size)
                .remark(param.getRemark())
                .operatorId(loginUser.getUserId())
                .operatorName(loginUser.getRealName())
                .sourceWarehouseId(param.getSourceWarehouseId())
                .targetWarehouseId(param.getTargetWarehouseId())
                .createTime(new Date())
                .build();
        inventoryOperationEntityMapper.insertSelective(inventoryOperationEntity);


        // 插入库存操作单据明细信息
        List<InventoryOperationDetailEntity> detailBuilderList = new ArrayList<>();
        for (InventoryOperationDetailParam detailParam : detailList) {
            InventoryOperationDetailEntity.InventoryOperationDetailEntityBuilder detailBuilder = InventoryOperationDetailEntity.builder();
            BigDecimal costPrice = productInventoryEntityMapper.selectCostPriceById(detailParam.getProductId());
            if (costPrice == null) {
                costPrice = BigDecimal.ZERO;
            }
            detailBuilder.docNo(docNo)
                    .productId(detailParam.getProductId())
                    .productName(detailParam.getProductName())
                    .productCode(detailParam.getProductCode())
                    .inventoryChangeQuantity(detailParam.getInventoryChangeQuantity())
                    .quantityBeforeUpdate(inventoryMap.get(detailParam.getProductId()))
                    .quantityAfterUpdate(inventoryMap.get(detailParam.getProductId()) + detailParam.getInventoryChangeQuantity())
                    .costPrice(costPrice)// 成本单价
                    .costPriceTotal(costPrice.multiply(new BigDecimal(detailParam.getInventoryChangeQuantity()))) // 成本总价
                    .remark(detailParam.getRemark());
            detailBuilderList.add(detailBuilder.build());
        }
        inventoryOperationDetailEntityMapper.insertList(detailBuilderList);
    }

    /**
     * 判断是否为入库操作
     * @param operationType 操作类型
     * @return 是否为入库操作
     */
    private boolean abs(Integer operationType) {
        InventoryOperationTypeEnum byCode = InventoryOperationTypeEnum.getByCode(operationType);
        if (byCode == null) {
            throw new SuperdeskBaseException(500, "操作类型异常");
        }
        boolean abs = true;
        switch (byCode) {
            case 调拨入库:
            case 采购入库:
            case 其他入库:
                break;
            case 调拨出库:
            case 采购退货:
            case 其他出库:
                abs = false;
                break;
        }
        return abs;
    }

    /**
     * 分页查询库存操作单据
     * @param param 查询参数
     * @return 库存操作单据分页结果
     */
    public PageResult<InventoryOperationVO> list(InventoryOperationPageQueryParam param) {
        AdminUtils.hasPermission(param.getOrgId());
        try (Page<InventoryOperationVO> ignored = PageHelper.startPage(param.getPageNo(), param.getPageSize())) {
            List<InventoryOperationVO> list = inventoryOperationEntityMapper.selectList(param);
            list.forEach(vo ->
                    vo.setOperationTypeName(InventoryOperationTypeEnum.getDescByCode(vo.getOperationType()))
            );
            PageInfo<InventoryOperationVO> pageInfo = new PageInfo<>(list);
            return new PageResult<>(param.getPageNo(), param.getPageSize(), pageInfo.getTotal(), pageInfo.getList());
        } catch (Exception e) {
            log.error("分页查询库存操作单据异常", e);
            throw new SuperdeskBaseException(500, "分页查询库存操作单据异常");
        }
    }

    /**
     * 查询库存操作单据详情
     * @param docNo 单据号
     * @return 库存操作单据详情
     */
    public InventoryOperationDetailInfoVO detail(String docNo) {
        InventoryOperationEntity inventoryOperationEntity = inventoryOperationEntityMapper.selectOne(
                InventoryOperationEntity.builder().docNo(docNo).build()
        );
        if (inventoryOperationEntity == null) {
            throw new SuperdeskBaseException(500, "单据号不存在");
        }
        AdminUtils.hasPermission(inventoryOperationEntity.getOrgId());

        List<InventoryOperationDetailEntity> detailEntityList = inventoryOperationDetailEntityMapper.select(
                InventoryOperationDetailEntity.builder()
                        .docNo(docNo)
                        .build()
        );

        if (CollUtil.isEmpty(detailEntityList)) {
            throw new SuperdeskBaseException(500, "单据明细不存在");
        }

        InventoryOperationVO inventoryOperationVO = InventoryOperationVO.builder()
                .orgName(inventoryOperationEntity.getOrgName())
                .docNo(inventoryOperationEntity.getDocNo())
                .merchantName(inventoryOperationEntity.getMerchantName())
                .operationTypeName(InventoryOperationTypeEnum.getDescByCode(inventoryOperationEntity.getOperationType()))
                .productTypeCount(inventoryOperationEntity.getProductTypeCount())
                .remark(inventoryOperationEntity.getRemark())
                .operatorName(inventoryOperationEntity.getOperatorName())
                .createTime(inventoryOperationEntity.getCreateTime())
                .build();

        List<InventoryOperationDetailVO> detailList = detailEntityList.stream()
                .map(detailEntity ->
                        InventoryOperationDetailVO.builder()
                                .productId(detailEntity.getProductId())
                                .productName(detailEntity.getProductName())
                                .productCode(detailEntity.getProductCode())
                                .inventoryChangeQuantity(detailEntity.getInventoryChangeQuantity())
                                .quantityBeforeUpdate(detailEntity.getQuantityBeforeUpdate())
                                .quantityAfterUpdate(detailEntity.getQuantityAfterUpdate())
                                .costPrice(detailEntity.getCostPrice())
                                .costPriceTotal(detailEntity.getCostPriceTotal())
                                .remark(detailEntity.getRemark())
                                .build()
                ).collect(Collectors.toList());

        return InventoryOperationDetailInfoVO.builder()
                .inventoryOperationVO(inventoryOperationVO)
                .detailList(detailList)
                .build();
    }

    /**
     * 导入商品库存单
     * @param file  文件
     * @param param 导入参数
     */
    @Transactional(rollbackFor = Exception.class)
    public void importInventory(MultipartFile file, InventoryOperationImportParam param) {
        LoginSession loginSession = AdminUtils.hasPermission(param.getOrgId());
        if (Objects.equals(param.getSourceWarehouseId(), param.getTargetWarehouseId())) {
            throw new SuperdeskBaseException(500, "调出仓库和调入仓库不能相同");
        }
        try {
            List<InventoryOperationExcelImportDTO> detailList = new ExcelReader(file.getInputStream()).parse(InventoryOperationExcelImportDTO.class);
            detailList.forEach(dto -> dto.setProductCode(dto.getProductCode().replaceAll("\\s*", "")));
            if (CollUtil.isEmpty(detailList)) {
                throw new SuperdeskBaseException(500, "导入数据为空");
            }
            log.info("导入商品库存单数据: {}", JSON.toJSONString(detailList));
            List<String> collect = detailList.stream().map(InventoryOperationExcelImportDTO::getProductCode).collect(Collectors.toList());
            Set<String> uniqueSet = new HashSet<>();
            Set<String> duplicateSet = collect.stream()
                    .filter(e -> !uniqueSet.add(e))
                    .collect(Collectors.toSet());
            if (CollUtil.isNotEmpty(duplicateSet)) {
                throw new SuperdeskBaseException(500, "导入数据中商品编码重复,重复编码: " + duplicateSet);
            }
            // 检查商品库存信息是否完整（排除多规格商品）
            List<ProductInventoryDTO> inventoryList = productInventoryEntityMapper.selectProductInventoryByConditions(uniqueSet, param.getOrgId(), param.getMerchantId());
            if (inventoryList.size() != uniqueSet.size()) {
                log.info("商品库存信息不完整,需要补足商品库存记录");
                List<String> existProductCodeList = inventoryList.stream().map(ProductInventoryDTO::getProductCode).collect(Collectors.toList());
                HashSet<String> copySet = new HashSet<>(uniqueSet);
                existProductCodeList.forEach(copySet::remove);
                log.info("需要补足库存信息的商品编码集合: {}", copySet);
                List<ProductInventoryEntity> insertList = new ArrayList<>();
                for (String productCode : copySet) {
                    Integer productId = productInventoryEntityMapper.selectProductIdByProductCode(productCode, param.getOrgId(), param.getMerchantId());
                    if (productId == null || productId == 0) {
                        throw new SuperdeskBaseException(500, "商品编码: " + productCode + " 不存在");
                    }
                    insertList.add(ProductInventoryEntity.builder()
                            .productId(productId)
                            .version(0)
                            .quantity(0)
                            .createdBy(loginSession.getUserId())
                            .createdName(loginSession.getUserName())
                            .createdAt(new Date())
                            .build());
                }
                productInventoryEntityMapper.insertList(insertList);
                // 补足后重新查询
                inventoryList = productInventoryEntityMapper.selectProductInventoryByConditions(uniqueSet, param.getOrgId(), param.getMerchantId());
            }

            // 转化为map key：商品条码 value：变动前库存数量
            Map<String, Integer> inventoryBeforeMap = inventoryList.stream()
                    .collect(Collectors.toMap(ProductInventoryDTO::getProductCode, ProductInventoryDTO::getQuantity));

            // 构建库存操作单据主体信息
            InventoryOperationEntity.InventoryOperationEntityBuilder builder = getInventoryOperationBuilder(param.getOrgId());

            // 商品变动数量map key：商品条码 value：变动数量
            Map<String, Integer> inventoryChangeMap = detailList.stream().collect(Collectors.toMap(
                    InventoryOperationExcelImportDTO::getProductCode,
                    InventoryOperationExcelImportDTO::getQuantity
            ));

            // 商品备注map key：商品条码 value：备注
            Map<String, String> inventoryRemarkMap = detailList.stream()
                    .filter(e -> StrUtil.isNotBlank(e.getRemark()))
                    .collect(Collectors.toMap(
                            InventoryOperationExcelImportDTO::getProductCode,
                            InventoryOperationExcelImportDTO::getRemark
                    ));
            boolean abs = abs(param.getOperationType());
            // 更新库存
            for (ProductInventoryDTO productInventoryDTO : inventoryList) {
                Integer change = inventoryChangeMap.get(productInventoryDTO.getProductCode());
                if (change == null) {
                    throw new SuperdeskBaseException(500, "商品库存变动数异常");
                }
                if (abs) {
                    change = Math.abs(change);
                } else {
                    change = -Math.abs(change);
                }
                int effect = productInventoryEntityMapper.updateInventory(productInventoryDTO.getId(), change, loginSession.getUserId(), loginSession.getUserName());
                if (effect != 1) {
                    throw new SuperdeskBaseException(500, "商品库存更新异常,商品编码: " + productInventoryDTO.getProductCode());
                }
            }

            // 生成单据号
            String docNo = DocNoUtils.generateDocNo("KC");

            // 插入库存操作单据主体信息
            InventoryOperationEntity inventoryOperationEntity = builder.docNo(docNo)
                    .merchantId(param.getMerchantId())
                    .merchantName(param.getMerchantName())
                    .operationType(param.getOperationType())
                    .productTypeCount(uniqueSet.size())
                    .remark(param.getRemark())
                    .operatorId(loginSession.getUserId())
                    .operatorName(loginSession.getRealName())
                    .createTime(new Date())
                    .sourceWarehouseId(param.getSourceWarehouseId())
                    .targetWarehouseId(param.getTargetWarehouseId())
                    .build();
            inventoryOperationEntityMapper.insertSelective(inventoryOperationEntity);


            // 插入库存操作单据明细信息
            List<InventoryOperationDetailEntity> detailBuilderList = new ArrayList<>();
            for (ProductInventoryDTO detailParam : inventoryList) {
                Integer before = inventoryBeforeMap.get(detailParam.getProductCode());// 变动前库存
                Integer change = inventoryChangeMap.get(detailParam.getProductCode());// 变动数量
                InventoryOperationDetailEntity.InventoryOperationDetailEntityBuilder detailBuilder = InventoryOperationDetailEntity.builder();

                BigDecimal costPrice = productInventoryEntityMapper.selectCostPriceById(detailParam.getProductId());
                if (costPrice == null) {
                    costPrice = BigDecimal.ZERO;
                }
                String remark = inventoryRemarkMap.get(detailParam.getProductCode());

                detailBuilder.docNo(docNo)
                        .productId(detailParam.getProductId())
                        .productName(detailParam.getProductName())
                        .productCode(detailParam.getProductCode())
                        .inventoryChangeQuantity(change)
                        .quantityBeforeUpdate(before)
                        .quantityAfterUpdate(before + change)
                        .costPrice(costPrice)
                        .costPriceTotal(costPrice.multiply(new BigDecimal(change)))
                        .remark(StrUtil.isBlank(remark) ? "" : remark.replaceAll("\\s*", ""))
                ;
                detailBuilderList.add(detailBuilder.build());
            }
            inventoryOperationDetailEntityMapper.insertList(detailBuilderList);
        } catch (SuperdeskBaseException e) {
            throw e;
        } catch (Exception e) {
            log.info("导入商品库存单异常", e);
            throw new SuperdeskBaseException(500, "导入商品库存单异常");
        }
    }

    private InventoryOperationEntity.InventoryOperationEntityBuilder getInventoryOperationBuilder(Integer orgId) {
        InventoryOperationEntity.InventoryOperationEntityBuilder builder = InventoryOperationEntity.builder();

        WrapperResponse<OrganizationEntityVO> projectInfoResponse = organizationFeignClient.getProjectInfoById(orgId);
        if (200 == projectInfoResponse.getCode()) {
            OrganizationEntityVO projectData = projectInfoResponse.getData();
            builder.compId(Integer.parseInt(projectData.getCompanyIds()))
                    .compName(projectData.getCompanyIdsName())
                    .orgId(orgId)
                    .orgName(projectData.getName());
        } else {
            throw new SuperdeskBaseException(500, "项目信息查询异常");
        }
        return builder;
    }

    /**
     * 导出商品库存单
     * @param docNo 单据号
     * @return 操作结果
     */
    public WrapperResponse<String> export(String docNo) {
        InventoryOperationEntity inventoryOperationEntity = inventoryOperationEntityMapper.selectOne(
                InventoryOperationEntity.builder()
                        .docNo(docNo)
                        .build()
        );
        if (inventoryOperationEntity == null) {
            throw new SuperdeskBaseException(500, "单据号不存在");
        }
        LoginSession loginSession = AdminUtils.hasPermission(inventoryOperationEntity.getOrgId());

        Map<String, Object> map = new HashMap<>();
        map.put("docNo", docNo);

        WrapperResponse<String> submit = exportFeignClient.submit(ExportTaskSubmitParam.builder()
                .platform(ExportTaskEnum.Platform.楼闪送.getCode())
                .module(ExportTaskEnum.Module.库存.getCode())
                .function(ExportInventoryFunctionConstants.导出库存操作单)
                .moduleName(ExportTaskEnum.Module.库存.getDesc())
                .source(ExportTaskEnum.Module.库存.getDesc())
                .exportFormat(ExportTaskEnum.Format.XLSX.getCode())
                .param(JSON.toJSONString(map))
                .artifactName("库存管理" + DateUtil.format(new Date(), "yyyyMMddHHmmss"))
                .createUserId(loginSession.getUserId())
                .createUserName(loginSession.getUserName())
                .compress(1)
                .build());
        log.info("导出商品库存单任务结果: {}, docNo: {}", JSON.toJSONString(submit), docNo);
        if (submit.getCode() != 200) {
            throw new SuperdeskBaseException(500, "导出任务提交失败");
        } else {
            return new WrapperResponse<>(200, "导出任务提交成功");
        }
    }
}
